// query = query.where('sharedTo', arrayContains: user.uid);

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:puma_firebase/entities/condition.dart';

Query<Map<String, dynamic>> queryConditions(
  Query<Map<String, dynamic>> query,
  List<PumaCondition> conditions,
) {
  for (PumaCondition condition in conditions) {
    query = queryCondition(query, condition);
  }

  return query;
}

Query<Map<String, dynamic>> queryCondition(
  Query<Map<String, dynamic>> query,
  PumaCondition condition,
) {
  switch (condition.type) {
    case PumaConditionType.isEqualTo:
      return query.where(condition.field, isEqualTo: condition.value);
    case PumaConditionType.isNotEqualTo:
      return query.where(condition.field, isNotEqualTo: condition.value);
    case PumaConditionType.isLessThan:
      return query.where(condition.field, isLessThan: condition.value);
    case PumaConditionType.isLessThanOrEqualTo:
      return query.where(condition.field, isLessThanOrEqualTo: condition.value);
    case PumaConditionType.isGreaterThan:
      return query.where(condition.field, isGreaterThan: condition.value);
    case PumaConditionType.arrayContains:
      return query.where(condition.field, arrayContains: condition.value);
    case PumaConditionType.arrayContainsAny:
      return query.where(condition.field, arrayContainsAny: condition.value);
    case PumaConditionType.whereIn:
      return query.where(condition.field, whereIn: condition.value);
    case PumaConditionType.whereNotIn:
      return query.where(condition.field, whereNotIn: condition.value);
    case PumaConditionType.isNull:
      return query.where(condition.field, isNull: condition.value);
    default:
      return query.limit(1);
  }
}

Query<Map<String, dynamic>> queryCollection(
  CollectionReference<Map<String, dynamic>> collection,
  PumaCondition condition,
) {
  switch (condition.type) {
    case PumaConditionType.isEqualTo:
      return collection.where(condition.field, isEqualTo: condition.value);
    case PumaConditionType.isNotEqualTo:
      return collection.where(condition.field, isNotEqualTo: condition.value);
    case PumaConditionType.isLessThan:
      return collection.where(condition.field, isLessThan: condition.value);
    case PumaConditionType.isLessThanOrEqualTo:
      return collection.where(condition.field,
          isLessThanOrEqualTo: condition.value);
    case PumaConditionType.isGreaterThan:
      return collection.where(condition.field, isGreaterThan: condition.value);
    case PumaConditionType.arrayContains:
      return collection.where(condition.field, arrayContains: condition.value);
    case PumaConditionType.arrayContainsAny:
      return collection.where(condition.field,
          arrayContainsAny: condition.value);
    case PumaConditionType.whereIn:
      return collection.where(condition.field, whereIn: condition.value);
    case PumaConditionType.whereNotIn:
      return collection.where(condition.field, whereNotIn: condition.value);
    case PumaConditionType.isNull:
      return collection.where(condition.field, isNull: condition.value);
    default:
      return collection.limit(1);
  }
}
